import logging
from typing import Any, Dict, List, Optional, Sequence, Union

from langchain_openai import ChatOpenAI
from langchain.callbacks.base import BaseCallbackHandler
from langchain.schema import LLMResult
from langchain_anthropic import ChatAnthropic

# Import necessary components
from .db_core import get_app_settings

DEFAULT_OPENAI_MODEL = "gpt-4o"
DEFAULT_ANTHROPIC_MODEL = "claude-3-haiku-20240307"

logger = logging.getLogger(__name__)

class TokenUsageCallbackHandler(BaseCallbackHandler):
    """Callback handler to track token usage."""

    def __init__(self):
        self.total_input_tokens = 0
        self.total_output_tokens = 0

    def reset(self) -> None:
        """Resets the token counts to zero."""
        self.total_input_tokens = 0
        self.total_output_tokens = 0

    def on_llm_start(
        self, serialized: Dict[str, Any], prompts: List[str], **kwargs: Any
    ) -> None:
        pass

    def on_llm_new_token(self, token: str, **kwargs: Any) -> None:
        self.total_output_tokens += 1

    def on_llm_end(self, response: LLMResult, **kwargs: Any) -> None:
        """Collect token usage from the LLMResult."""
        # Access token usage safely from llm_output dictionary
        llm_output = response.llm_output or {}
        token_usage = llm_output.get('token_usage', {})
        self.total_input_tokens += token_usage.get('prompt_tokens', 0)
        self.total_output_tokens += token_usage.get('completion_tokens', 0)

    def on_llm_error(self, error: Exception, **kwargs: Any) -> None:
        logger.error(f"LLM error: {error}")

def initialize_llm(
    provider: str = 'openai',
    model_name: Optional[str] = None,
    temperature: float = 0.7,
    api_key_override: Optional[str] = None
) -> tuple[Union[ChatOpenAI, ChatAnthropic], TokenUsageCallbackHandler]:
    """
    Initializes the specified LLM provider (OpenAI or Anthropic) with API key handling.

    Args:
        provider: The LLM provider ('openai' or 'anthropic'). Defaults to 'openai'.
        model_name: The specific model name to use. Defaults to provider's default.
        temperature: The sampling temperature for the model.
        api_key_override: An API key to use instead of the one from settings.

    Returns:
        A tuple containing the initialized LLM instance and the TokenUsageCallbackHandler instance.

    Raises:
        ValueError: If the API key is missing or the provider is unsupported.
    """
    token_callback = TokenUsageCallbackHandler()
    app_settings = get_app_settings()

    if provider == 'openai':
        api_key = api_key_override if api_key_override else app_settings.get('openai')
        if not api_key:
            logger.error("OpenAI API key not found (checked override and settings).")
            raise ValueError("Missing OpenAI API Key")
        model = model_name or DEFAULT_OPENAI_MODEL
        llm = ChatOpenAI(model=model, temperature=temperature, api_key=api_key)
        logger.info(f"Initialized ChatOpenAI with model: {model}")

    elif provider == 'anthropic':
        api_key = api_key_override if api_key_override else app_settings.get('anthropic')
        if not api_key:
            logger.error("Anthropic API key not found (checked override and settings).")
            raise ValueError("Missing Anthropic API Key")
        model = model_name or DEFAULT_ANTHROPIC_MODEL
        llm = ChatAnthropic(model=model, temperature=temperature, api_key=api_key, max_tokens=2048) # Add max_tokens for Anthropic
        logger.info(f"Initialized ChatAnthropic with model: {model}")

    else:
        logger.error(f"Unsupported LLM provider specified: {provider}")
        raise ValueError(f"Unsupported LLM provider: {provider}. Choose 'openai' or 'anthropic'.")

    # Return both the LLM instance and the callback handler
    return llm, token_callback
